@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ####################################################################################################
-# 
+#
 # PySpice - A Spice Package for Python
 # Copyright (C) 2014 Fabrice Salvaire
 #
@@ -9,15 +9,15 @@
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>. 
-# 
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 ####################################################################################################
 
 """ This module implements high level elements built on Spice elements. """
@@ -87,7 +87,7 @@ class Sinusoidal(VoltageSource):
                  offset=0, amplitude=1, frequency=50,
                  delay=0, damping_factor=0):
 
-        super(Sinusoidal, self).__init__(name, node_plus, node_minus)
+        super().__init__(name, node_plus, node_minus)
 
         self.dc_offset = dc_offset
         self.offset = offset
@@ -116,7 +116,7 @@ class Sinusoidal(VoltageSource):
                           'AC SIN({}V {}V {}Hz {}s {})'.format(self.offset, self.amplitude,
                                                                self.frequency, self.delay,
                                                                self.damping_factor)))
-        
+
 ####################################################################################################
 
 class AcLine(Sinusoidal):
@@ -125,7 +125,7 @@ class AcLine(Sinusoidal):
 
     def __init__(self, name, node_plus, node_minus, rms_voltage=230, frequency=50):
 
-        super(AcLine, self).__init__(name, node_plus, node_minus,
+        super().__init__(name, node_plus, node_minus,
                                      amplitude=rms_to_amplitude(rms_voltage),
                                      frequency=frequency)
 
@@ -156,11 +156,11 @@ class Pulse(VoltageSource):
     +--------+---------------+---------------+-------+
 
     Spice Syntax::
-    
+
         PULSE ( V1 V2 Td Tr Tf Pw Period )
-    
+
     A single pulse so specified is described by the following table:
-    
+
     +-------------+-------+
     | Time        | Value |
     +-------------+-------+
@@ -176,7 +176,7 @@ class Pulse(VoltageSource):
     +-------------+-------+
     | Tstop       | V1    |
     +-------------+-------+
-    
+
     Note: default value in Spice for rise and fall time is the simulation transient step, pulse
     width and period is the simulation stop time.
 
@@ -209,7 +209,7 @@ class Pulse(VoltageSource):
         #  rise_time, fall_time = Tstep
         #  pulse_width, period = Tstop
 
-        super(Pulse, self).__init__(name, node_plus, node_minus)
+        super().__init__(name, node_plus, node_minus)
 
         self.initial_value = initial_value
         self.pulsed_value = pulsed_value
@@ -271,15 +271,15 @@ class Exponential(VoltageSource):
     +------+--------------------+---------------+-------+
 
     Spice Syntax::
-    
+
         EXP ( V1 V2 TD1 TAU1 TD2 TAU2 )
-    
+
     The shape of the waveform is described by the following formula:
 
     Let V21 = V2 - V1 and V12 = V1 - V2.
-   
+
     .. math::
-   
+
         V(t) = \begin{cases}
           V_1 & \text{if}\ 0 \leq t < T_{d1}, \\
           V_1 + V_{21} ( 1 − e^{-\frac{t-T_{d1}}{\tau_1}} )
@@ -300,7 +300,7 @@ class Exponential(VoltageSource):
 
         # Fixme: default
 
-        super(Exponential, self).__init__(name, node_plus, node_minus)
+        super().__init__(name, node_plus, node_minus)
 
         self.initial_value = initial_value
         self.pulsed_value = pulsed_value
@@ -308,7 +308,7 @@ class Exponential(VoltageSource):
         self.rise_time_constant = rise_time_constant
         self.fall_delay_time = fall_delay_time
         self.fall_time_constant = fall_time_constant
- 
+
     ##############################################
 
     def format_spice_parameters(self):
@@ -352,12 +352,12 @@ class PieceWiseLinear(VoltageSource):
 
         # Fixme: default
 
-        super(PieceWiseLinear, self).__init__(name, node_plus, node_minus)
+        super().__init__(name, node_plus, node_minus)
 
         self.values = values
         self.repeate_time = repeate_time
         self.delay_time = delay_time
- 
+
     ##############################################
 
     def format_spice_parameters(self):
@@ -405,7 +405,7 @@ class SingleFrequencyFM(VoltageSource):
     def __init__(self, name, node_plus, node_minus,
                  offset, amplitude, carrier_frequency, modulation_index, signal_frequency):
 
-        super(SingleFrequencyFM, self).__init__(name, node_plus, node_minus)
+        super().__init__(name, node_plus, node_minus)
 
         self.offset = offset
         self.amplitude = amplitude
@@ -442,15 +442,15 @@ class AmplitudeModulated(VoltageSource):
     +------+----------------------+---------------+-------+
     | Td   + signal delay         +               + s     |
     +------+----------------------+---------------+-------+
-    
+
     Spice Syntax::
-    
+
         AM(VA VO MF FC TD)
-    
+
     The shape of the waveform is described by the following equation:
 
     .. math::
-  
+
         V(t) = V_a (V_o + \sin (2\pi M_f\,t)) \sin (2\pi F_c\,t)
 
     """
@@ -462,14 +462,14 @@ class AmplitudeModulated(VoltageSource):
 
         # Fixme: default
 
-        super(AmplitudeModulated, self).__init__(name, node_plus, node_minus)
+        super().__init__(name, node_plus, node_minus)
 
         self.offset = offset
         self.amplitude = amplitude
         self.carrier_frequency = Frequency(carrier_frequency)
         self.modulating_frequency = Frequency(modulating_frequency)
         self.signal_delay = signal_delay
- 
+
     ##############################################
 
     def format_spice_parameters(self):
@@ -492,11 +492,11 @@ class RandomVoltage(VoltageSource):
     control of behavioral sources (B, E, G sources, voltage controllable A sources, capacitors,
     inductors, or resistors) to simulate the circuit dependence on statistically varying device
     parameters. A Monte-Carlo simulation may thus be handled in a single simulation run.
-    
+
     Spice Syntax::
-    
+
         TRRANDOM( TYPE TS <TD <PARAM1 <PARAM2> > >)
-    
+
     TYPE determines the random variates generated: 1 is uniformly distributed, 2 Gaussian, 3
     exponential, 4 Poisson. TS is the duration of an individual voltage value. TD is a time delay
     with 0 V output before the random voltage values start up. PARAM1 and PARAM2 depend on the type
@@ -523,8 +523,8 @@ class RandomVoltage(VoltageSource):
 
         # Fixme: random_type and parameters
 
-        super(RandomVoltage, self).__init__(name, node_plus, node_minus)
- 
+        super().__init__(name, node_plus, node_minus)
+
         self.random_type = random_type
         self.duration = duration
         self.time_delay = time_delay
@@ -553,7 +553,7 @@ class RandomVoltage(VoltageSource):
                 ')')
 
 ####################################################################################################
-# 
+#
 # End
-# 
+#
 ####################################################################################################
