@@ -1,5 +1,5 @@
 ####################################################################################################
-# 
+#
 # PySpice - A Spice Package for Python
 # Copyright (C) 2014 Fabrice Salvaire
 #
@@ -7,15 +7,15 @@
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>. 
-# 
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+#
 ####################################################################################################
 
 """This modules implements circuit and subcircuit.
@@ -46,7 +46,7 @@ or as a class definition:
 
           def __init__(self, **kwargs):
 
-              super(VoltageDivider, self).__init__(title='Voltage Divider', **kwargs)
+              super().__init__(title='Voltage Divider', **kwargs)
 
               self.V('input', 'in', self.gnd, '10V')
               self.R(1, 'in', 'out', kilo(9))
@@ -69,6 +69,10 @@ We can update an element parameter like this::
 
     circuit.R1.resistance = kilo(1)
 
+To simulate the circuit, we must create a simulator instance using the :meth:`Circuit.simulator`::
+
+    simulator = circuit.simulator()
+
 """
 
 ####################################################################################################
@@ -87,6 +91,9 @@ We can update an element parameter like this::
 ####################################################################################################
 
 from collections import OrderedDict
+import keyword
+import logging
+import os
 
 # import networkx
 
@@ -96,11 +103,15 @@ from ..Tools.StringTools import join_lines, join_list, join_dict
 from .ElementParameter import (ParameterDescriptor,
                                PositionalElementParameter,
                                FlagParameter, KeyValueParameter)
-from .Simulation import CircuitSimulator
+from .Simulation import SubprocessCircuitSimulator, NgSpiceSharedCircuitSimulator
 
 ####################################################################################################
 
-class DeviceModel(object):
+_module_logger = logging.getLogger(__name__)
+
+####################################################################################################
+
+class DeviceModel:
 
     """ This class implements a device model.
 
@@ -178,16 +189,21 @@ class DeviceModel(object):
 
 ####################################################################################################
 
-class Pin(object):
+class Pin:
 
     """This class implements a pin of an element. It stores a reference to the element, the name of the
     pin and the node.
     """
 
+    _logger = _module_logger.getChild('Pin')
+
     ##############################################
 
     def __init__(self, element, name, node):
 
+        if keyword.iskeyword(node):
+            self._logger.warning("Node {} is a Python keyword".format(node))
+
         self._element = element
         self._name = name
         self._node = node # Fixme: name, not a Node instance, cf. Netlist.nodes
@@ -220,9 +236,10 @@ class Pin(object):
 
     def add_current_probe(self, circuit):
 
-        """ Add a current probe between the node and the pin.
+        """Add a current probe between the node and the pin.
 
         The ammeter is named *ElementName_PinName*.
+
         """
 
         # Fixme: require a reference to circuit
@@ -238,40 +255,109 @@ class ElementParameterMetaClass(type):
 
     """ Metaclass to implements the element parameter machinery. """
 
+    __classes__ = {}
+
     ##############################################
 
-    def __new__(cls, name, bases, attributes):
+    def __new__(cls, class_name, super_classes, attributes):
 
-        sorted_attributes = OrderedDict(sorted(attributes.items(), key=lambda t: t[0]))
-
-        positional_parameters = OrderedDict()
-        parameters = OrderedDict() # not required for SPICE, but for unit test
-        for attribute_name, obj in sorted_attributes.iteritems():
+        # Collect positional and optional parameters from class attribute dict
+        positional_parameters = {}
+        parameters = {}
+        for attribute_name, obj in attributes.items():
             if isinstance(obj, ParameterDescriptor):
                 obj.attribute_name = attribute_name
                 if isinstance(obj, PositionalElementParameter):
                     d = positional_parameters
                 elif isinstance(obj, (FlagParameter, KeyValueParameter)):
                     d = parameters
+                # else:
+                #     raise NotImplementedError
                 d[attribute_name] = obj
-        attributes['positional_parameters'] = positional_parameters
-        attributes['optional_parameters'] = parameters
-        attributes['parameters_from_args'] = [parameter
-                                              for parameter in sorted(positional_parameters.itervalues())
-                                              if not parameter.key_parameter]
 
-        return super(ElementParameterMetaClass, cls).__new__(cls, name, bases, attributes)
+        # Build dictionnary : attribute_name -> parameter
+        attributes['_positional_parameters'] = OrderedDict(sorted(list(positional_parameters.items()),
+                                                                  key=lambda t: t[1].position))
+
+        # optional parameter order is not required for SPICE, but for unit test
+        attributes['_optional_parameters'] = OrderedDict(sorted(list(parameters.items()), key=lambda t: t[0]))
+
+        # Positional parameter array
+        attributes['_parameters_from_args'] = [parameter
+                                               for parameter in sorted(positional_parameters.values())
+                                               if not parameter.key_parameter]
+
+        # Implement alias for parameters
+        attributes['_spice_to_parameters'] = {parameter.spice_name:parameter
+                                              for parameter in attributes['_optional_parameters'].values()}
+        for parameter in attributes['_spice_to_parameters'].values():
+            if (parameter.spice_name in attributes
+                and parameter.spice_name != parameter.attribute_name):
+                _module_logger.error('Spice parameter "{}" clash with attributes'.format(parameter.spice_name))
+
+        return super().__new__(cls, class_name, super_classes, attributes)
+
+    ##############################################
+
+    def __init__(cls, class_name, super_classes, attributes):
+
+        type.__init__(cls, class_name, super_classes, attributes)
+
+        # Collect basic element classes
+        if 'prefix' in attributes:
+            prefix = attributes['prefix']
+            if prefix is not None:
+                classes = ElementParameterMetaClass.__classes__
+                if prefix in classes:
+                    classes[prefix].append(cls)
+                else:
+                    classes[prefix] = [cls]
+
+    ##############################################
+
+    # Notes: These properties are only accessible from the class object, e.g. instance.__class__
+
+    @property
+    def number_of_pins(cls):
+        return cls._number_of_pins
+
+    @property
+    def number_of_positional_parameters(cls):
+        return len(cls._positional_parameters)
+
+    @property
+    def positional_parameters(cls):
+        return cls._positional_parameters
+
+    @property
+    def optional_parameters(cls):
+        return cls._optional_parameters
+
+    @property
+    def parameters_from_args(cls):
+        return cls._parameters_from_args
+
+    @property
+    def spice_to_parameters(cls):
+        return cls._spice_to_parameters
 
 ####################################################################################################
 
-class Element(object):
+class Element(metaclass=ElementParameterMetaClass):
 
     """ This class implements a base class for an element.
 
     It use a metaclass machinery for the declaration of the parameters.
     """
 
-    __metaclass__ = ElementParameterMetaClass
+    # These attributes are defined in subclasses or via the metaclass.
+    _number_of_pins = None
+    _positional_parameters = None
+    _optional_parameters = None
+    _parameters_from_args = None
+    _spice_to_parameters = None
+
+    # Fixme: _prefix
 
     #: SPICE element prefix
     prefix = None
@@ -285,10 +371,10 @@ class Element(object):
 
         # self._parameters = list(args)
 
-        for parameter, value in zip(self.parameters_from_args, args):
+        for parameter, value in zip(self._parameters_from_args, args):
             setattr(self, parameter.attribute_name, value)
-        for key, value in kwargs.iteritems():
-            if key in self.positional_parameters or self.optional_parameters:
+        for key, value in kwargs.items():
+            if key in self._positional_parameters or self._optional_parameters:
                 setattr(self, key, value)
 
     ##############################################
@@ -297,8 +383,6 @@ class Element(object):
     def name(self):
         return self.prefix + self._name
 
-    ##############################################
-
     @property
     def pins(self):
         return self._pins
@@ -312,11 +396,32 @@ class Element(object):
     ##############################################
 
     def __repr__(self):
-
         return self.__class__.__name__ + ' ' + self.name
 
     ##############################################
 
+    def __setattr__(self, name, value):
+
+        # Implement alias for parameters
+        if name in self._spice_to_parameters:
+            parameter = self._spice_to_parameters[name]
+            object.__setattr__(self, parameter.attribute_name, value)
+        else:
+            object.__setattr__(self, name, value)
+
+    ##############################################
+
+    def __getattr__(self, name):
+
+        # Implement alias for parameters
+        if name in self._spice_to_parameters:
+            parameter = self._spice_to_parameters[name]
+            return object.__getattribute__(self, parameter.attribute_name)
+        else:
+            raise AttributeError(name)
+
+    ##############################################
+
     def format_node_names(self):
         """ Return the formatted list of nodes. """
         return join_list((self.name, join_list(self.nodes)))
@@ -325,8 +430,8 @@ class Element(object):
 
     def parameter_iterator(self):
         """ This iterator returns the parameter in the right order. """
-        for parameter_dict in self.positional_parameters, self.optional_parameters:
-            for parameter in parameter_dict.itervalues():
+        for parameter_dict in self._positional_parameters, self._optional_parameters:
+            for parameter in parameter_dict.values():
                 if parameter.nonzero(self):
                     yield parameter
 
@@ -350,17 +455,31 @@ class Element(object):
 
 ####################################################################################################
 
+class NPinElement(Element):
+    pass
+
+####################################################################################################
+
+class AnyPinElement(Element):
+    _number_of_pins = 0
+
+####################################################################################################
+
 class TwoPinElement(Element):
 
     """ This class implements a base class for a two-pin element. """
 
+    # dipole
+
+    _number_of_pins = 2
+
     ##############################################
 
     def __init__(self, name, node_plus, node_minus, *args, **kwargs):
 
         pins = (Pin(self, 'plus', node_plus), Pin(self, 'minus', node_minus))
 
-        super(TwoPinElement, self).__init__(name, pins, *args, **kwargs)
+        super().__init__(name, pins, *args, **kwargs)
 
     ##############################################
 
@@ -376,6 +495,16 @@ class TwoPinElement(Element):
 
 ####################################################################################################
 
+class ThreePinElement(Element):
+    _number_of_pins = 3
+
+####################################################################################################
+
+class FourPinElement(Element):
+    _number_of_pins = 4
+
+####################################################################################################
+
 class TwoPortElement(Element):
 
     """ This class implements a base class for a two-port element.
@@ -383,8 +512,12 @@ class TwoPortElement(Element):
     .. warning:: As opposite to Spice, the input nodes are specified before the output nodes.
     """
 
+    _number_of_pins = 4
+
     ##############################################
 
+    # Fixme: Why the order the inverted ?
+
     def __init__(self, name,
                  input_node_plus, input_node_minus,
                  output_node_plus, output_node_minus,
@@ -395,7 +528,7 @@ class TwoPortElement(Element):
                 Pin(self, 'input_plus', input_node_plus),
                 Pin(self, 'input_minus', input_node_minus))
 
-        super(TwoPortElement, self).__init__(name, pins, *args, **kwargs)
+        super().__init__(name, pins, *args, **kwargs)
 
     ##############################################
 
@@ -423,7 +556,7 @@ class TwoPortElement(Element):
 
 ####################################################################################################
 
-class Node(object):
+class Node:
 
     """This class implements a node in the circuit. It stores a reference to the elements connected to
     the node."""
@@ -434,7 +567,7 @@ class Node(object):
 
     def __init__(self, name):
 
-        self._name = name
+        self._name = str(name)
         self._elements = set()
 
     ##############################################
@@ -455,11 +588,11 @@ class Node(object):
     ##############################################
 
     def add_element(self, element):
-        self._elements.add(element) 
+        self._elements.add(element)
 
 ####################################################################################################
 
-class Netlist(object):
+class Netlist:
 
     """ This class implements a base class for a netlist.
 
@@ -483,13 +616,19 @@ class Netlist(object):
 
     def element_iterator(self):
 
-        return self._elements.itervalues()
+        return iter(self._elements.values())
+
+    ##############################################
+
+    def element_names(self):
+
+        return [element.name for element in self.element_iterator()]
 
     ##############################################
 
     def model_iterator(self):
 
-        return self._models.itervalues()
+        return iter(self._models.values())
 
     ##############################################
 
@@ -506,7 +645,7 @@ class Netlist(object):
 
     def _add_element(self, element):
 
-        """ Add an element. """
+        """Add an element."""
 
         if element.name not in self._elements:
             self._elements[element.name] = element
@@ -518,7 +657,7 @@ class Netlist(object):
 
     def model(self, name, modele_type, **parameters):
 
-        """ Add a model. """
+        """Add a model."""
 
         model = DeviceModel(name, modele_type, **parameters)
         if model.name not in self._models:
@@ -531,7 +670,7 @@ class Netlist(object):
     @property
     def nodes(self):
 
-        """ Return the nodes. """
+        """Return the nodes."""
 
         if self._dirty:
             # nodes = set()
@@ -549,7 +688,13 @@ class Netlist(object):
                     else:
                         node = self._nodes[node_name]
                     node.add_element(element)
-        return self._nodes.values()
+        return list(self._nodes.values())
+
+    ##############################################
+
+    def node_names(self):
+
+        return [node.name for node in self.nodes]
 
     ##############################################
 
@@ -567,7 +712,7 @@ class Netlist(object):
     ##############################################
 
     def __getattr__(self, attribute_name):
-        
+
         try:
             return self.__getitem__(attribute_name)
         except IndexError:
@@ -577,18 +722,22 @@ class Netlist(object):
 
 class SubCircuit(Netlist):
 
-    """ This class implements a sub-cicuit netlist. """
+    """This class implements a sub-cicuit netlist."""
 
     ##############################################
 
     def __init__(self, name, *nodes, **kwargs):
 
-        super(SubCircuit, self).__init__()
+        super().__init__()
 
         self.name = str(name)
         self._external_nodes = list(nodes)
+        # Fixme: ok ?
         self._ground = kwargs.get('ground', 0)
-        
+        if 'ground' in kwargs:
+            del kwargs['ground']
+        self._parameters = kwargs
+
     ##############################################
 
     @property
@@ -598,9 +747,16 @@ class SubCircuit(Netlist):
 
     ##############################################
 
+    @property
+    def parameters(self):
+        """Parameters"""
+        return self._parameters
+
+    ##############################################
+
     def check_nodes(self):
 
-        """ Check for dangling nodes in the subcircuit. """
+        """Check for dangling nodes in the subcircuit."""
 
         nodes = set(self._external_nodes)
         connected_nodes = set()
@@ -614,11 +770,14 @@ class SubCircuit(Netlist):
 
     def __str__(self):
 
-        """ Return the formatted subcircuit definition. """
+        """Return the formatted subcircuit definition."""
 
-        netlist = '.subckt {} {}\n'.format(self.name, join_list(self._external_nodes))
-        netlist += super(SubCircuit, self).__str__()
-        netlist += '.ends\n'
+        nodes = join_list(self._external_nodes)
+        parameters = join_list(['{}={}'.format(key, value)
+                                for key, value in self._parameters.items()])
+        netlist = '.subckt ' + join_list((self.name, nodes, parameters)) + '\n'
+        netlist += super().__str__()
+        netlist += '.ends ' + self.name + '\n'
         return netlist
 
 ####################################################################################################
@@ -634,13 +793,13 @@ class SubCircuitFactory(SubCircuit):
 
     def __init__(self, **kwargs):
 
-        super(SubCircuitFactory, self).__init__(self.__name__, *self.__nodes__, **kwargs)
+        super().__init__(self.__name__, *self.__nodes__, **kwargs)
 
 ####################################################################################################
 
 class Circuit(Netlist):
 
-    """ This class implements a cicuit netlist.
+    """This class implements a cicuit netlist.
 
     To get the corresponding Spice netlist use::
 
@@ -661,12 +820,12 @@ class Circuit(Netlist):
                  global_nodes=(),
              ):
 
-        super(Circuit, self).__init__()
+        super().__init__()
 
         self.title = str(title)
         self._ground = ground
         self._global_nodes = set(global_nodes) # .global
-        self._includes = set() # .include
+        self._includes = [] # .include
         self._parameters = {} # .param
         self._subcircuits = {}
 
@@ -685,23 +844,26 @@ class Circuit(Netlist):
 
     def include(self, path):
 
-        """ Include a file. """
+        """Include a file."""
 
-        self._includes.add(path)
+        if path not in self._includes:
+            self._includes.append(path)
+        else:
+            self._logger.warn("Duplicated include")
 
     ##############################################
 
     def parameter(self, name, expression):
 
-        """ Set a parameter. """
-        
+        """Set a parameter."""
+
         self._parameters[str(name)] = str(expression)
 
     ##############################################
 
     def subcircuit(self, subcircuit):
 
-        """ Add a sub-circuit. """
+        """Add a sub-circuit."""
 
         self._subcircuits[str(subcircuit.name)] = subcircuit
 
@@ -709,37 +871,62 @@ class Circuit(Netlist):
 
     def subcircuit_iterator(self):
 
-        """ Return a sub-circuit iterator. """
+        """Return a sub-circuit iterator."""
 
-        return self._subcircuits.itervalues()
+        return iter(self._subcircuits.values())
 
     ##############################################
 
     def __str__(self):
 
-        """ Return the formatted desk. """
+        """Return the formatted desk."""
 
         netlist = '.title {}\n'.format(self.title)
         if self._includes:
-            netlist += join_lines(self._includes, prefix='.include ')  + '\n'
+            # ngspice don't like // in path, thus ensure we write real paths
+            real_paths = [os.path.realpath(str(path)) for path in self._includes]
+            netlist += join_lines(real_paths, prefix='.include ')  + '\n'
         if self._global_nodes:
             netlist += '.global ' + join_list(self._global_nodes) + '\n'
         if self._parameters:
             netlist += join_lines(self._parameters, prefix='.param ') + '\n'
         if self._subcircuits:
             netlist += join_lines(self.subcircuit_iterator())
-        netlist += super(Circuit, self).__str__()
-        netlist += '.end\n'
+        netlist += super().__str__()
         return netlist
 
     ##############################################
 
+    def str_end(self):
+
+        return str(self) + '.end\n'
+
+    ##############################################
+
     def simulator(self, *args, **kwargs):
-        """ Return a :obj:`PySpice.Simulation.CircuitSimulator` instance. """
-        return CircuitSimulator(self, *args, **kwargs)
+
+        """Return a :obj:`PySpice.Spice.Simulation.SubprocessCircuitSimulator` or
+        :obj:`PySpice.Spice.Simulation.NgSpiceSharedCircuitSimulator` instance depending of the
+        value of the *simulator* parameter: ``subprocess`` or ``shared``, respectively. If this
+        parameter is not specified then a subprocess simulator is returned.
+
+        """
+
+        simulator = kwargs.get('simulator', 'subprocess')
+        if 'simulator' in kwargs:
+            simulator = kwargs['simulator']
+            del kwargs['simulator']
+        else:
+            simulator = 'subprocess'
+        if simulator == 'subprocess':
+            return SubprocessCircuitSimulator(self, *args, **kwargs)
+        elif simulator == 'shared':
+            return NgSpiceSharedCircuitSimulator(self, *args, **kwargs)
+        else:
+            return ValueError('Unknown simulator type')
 
 ####################################################################################################
-# 
+#
 # End
-# 
+#
 ####################################################################################################
