@@ -22,11 +22,11 @@ simulator = circuit.simulator(temperature=25, nominal_temperature=25)
 
 analysis = simulator.operating_point()
 for node in (analysis['in'], analysis.out): # .in is invalid !
-    print 'Node {}: {} V'.format(str(node), float(node))
+    print('Node {}: {} V'.format(str(node), float(node)))
 
 analysis = simulator.dc_sensitivity('v(out)')
-for element in analysis.elements.itervalues():
-    print element, float(element)
+for element in analysis.elements.values():
+    print(element, float(element))
 
 ####################################################################################################
 # 
